@@ -1,25 +1,40 @@
-/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *   Ben Goodger <ben@netscape.com> (Original Author)
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 var gBookmarksShell = null;
 
@@ -533,18 +548,11 @@ BookmarksTree.prototype = {
 
   treeOpen: function (aEvent)
   {
-    if (!this.isValidOpenEvent(aEvent))
-      return;
-
-    var rdfNode = this.findRDFNode(aEvent.target, true);
-    if (rdfNode.getAttribute("container") == "true") {
-      if (this.openClickCount == 1)
-        rdfNode.setAttribute("open", rdfNode.getAttribute("open") != "true");
-      gSelectionTracker.clickCount = 0;
-      return;
+    if (this.isValidOpenEvent(aEvent)) {
+      var rdfNode = this.findRDFNode(aEvent.target, true);
+      if (rdfNode.getAttribute("container") != "true")
+        this.open(aEvent, rdfNode);
     }
-
-    this.open(aEvent, rdfNode);
   },
 
   /////////////////////////////////////////////////////////////////////////////
@@ -622,6 +630,7 @@ BookmarksTree.prototype = {
     isCommandEnabled: function (aCommand)
     {
       var numSelectedItems = gBookmarksShell.tree.selectedItems.length;
+      var seln, firstSelected, folderType, bItemCountCorrect;
       switch(aCommand) {
       case "cmd_undo":
       case "cmd_redo":
@@ -635,7 +644,7 @@ BookmarksTree.prototype = {
       case "cmd_bm_selectAll":
         return true;
       case "cmd_open":
-        var seln = gBookmarksShell.tree.selectedItems;
+         seln = gBookmarksShell.tree.selectedItems;
         return numSelectedItems == 1 && seln[0].getAttribute("type") == NC_NS + "Bookmark";
       case "cmd_openfolder":
       case "cmd_openfolderinnewwindow":
@@ -654,24 +663,24 @@ BookmarksTree.prototype = {
         return numSelectedItems == 1 && seln[0].getAttribute("type") != NC_NS + "BookmarkSeparator";
       case "cmd_setnewbookmarkfolder":
         seln = gBookmarksShell.tree.selectedItems;
-        var firstSelected = seln.length ? seln[0] : gBookmarksShell.tree;
-        var folderType = firstSelected.getAttribute("type") == (NC_NS + "Folder");
-        var bItemCountCorrect = seln.length ? numSelectedItems == 1 : true;
+        firstSelected = seln.length ? seln[0] : gBookmarksShell.tree;
+        folderType = firstSelected.getAttribute("type") == (NC_NS + "Folder");
+        bItemCountCorrect = seln.length ? numSelectedItems == 1 : true;
         return bItemCountCorrect && !(NODE_ID(firstSelected) == "NC:NewBookmarkFolder") && folderType;
       case "cmd_setpersonaltoolbarfolder":
         seln = gBookmarksShell.tree.selectedItems;
-        var firstSelected = seln.length ? seln[0] : gBookmarksShell.tree;
-        var folderType = firstSelected.getAttribute("type") == (NC_NS + "Folder");
-        var bItemCountCorrect = seln.length ? numSelectedItems == 1 : true;
+        firstSelected = seln.length ? seln[0] : gBookmarksShell.tree;
+        folderType = firstSelected.getAttribute("type") == (NC_NS + "Folder");
+        bItemCountCorrect = seln.length ? numSelectedItems == 1 : true;
         return bItemCountCorrect && !(NODE_ID(firstSelected) == "NC:PersonalToolbarFolder") && folderType;
       case "cmd_setnewsearchfolder":
         seln = gBookmarksShell.tree.selectedItems;
-        var firstSelected = seln.length ? seln[0] : gBookmarksShell.tree;
-        var folderType = firstSelected.getAttribute("type") == (NC_NS + "Folder");
-        var bItemCountCorrect = seln.length ? numSelectedItems == 1 : true;
+        firstSelected = seln.length ? seln[0] : gBookmarksShell.tree;
+        folderType = firstSelected.getAttribute("type") == (NC_NS + "Folder");
+        bItemCountCorrect = seln.length ? numSelectedItems == 1 : true;
         return bItemCountCorrect == 1 && !(NODE_ID(firstSelected) == "NC:NewSearchFolder") && folderType;
       case "cmd_bm_fileBookmark":
-        var seln = gBookmarksShell.tree.selectedItems;
+        seln = gBookmarksShell.tree.selectedItems;
         return seln.length > 0;
       default:
         return false;
