@@ -40,6 +40,9 @@ var messengerMigratorContractID   = "@mozilla.org/messenger/migrator;1";
 var msgComposeService = Components.classes["@mozilla.org/messengercompose;1"].getService();
 msgComposeService = msgComposeService.QueryInterface(Components.interfaces.nsIMsgComposeService);
 
+// We are storing the value of the bool logComposePerformance inorder to avoid logging unnecessarily.
+var gLogComposePerformance = msgComposeService.logComposePerformance;
+
 var gPromptService = null;
 
 //This migrates the LDAPServer Preferences from 4.x to mozilla format.
@@ -53,7 +56,7 @@ try {
 var msgCompose = null;
 var MAX_RECIPIENTS = 0;
 var currentAttachment = null;
-var windowLocked = false;
+var gWindowLocked = false;
 var contentChanged = false;
 var gCurrentIdentity = null;
 var defaultSaveOperation = "draft";
@@ -65,6 +68,7 @@ var gAutocompleteSession = null;
 var gSetupLdapAutocomplete = false;
 var gLDAPSession = null;
 var gComposeMsgsBundle;
+var gSavedSendNowKey;
 
 const DEBUG = false;
 
@@ -86,7 +90,6 @@ if (prefs) {
 function disableEditableFields()
 {
   editorShell.editor.SetFlags(plaintextEditor.eEditorReadonlyMask);
-  document.getElementById("msgSubject").setAttribute('disabled', 'true');
   var disableElements = document.getElementsByAttribute("disableonsend", "true");
   for (i=0;i<disableElements.length;i++)
   {
@@ -97,7 +100,6 @@ function disableEditableFields()
 function enableEditableFields()
 {
   editorShell.editor.SetFlags(plaintextEditor.eEditorMailMask);
-  document.getElementById("msgSubject").removeAttribute("disabled");
   var enableElements = document.getElementsByAttribute("disableonsend", "true");
   for (i=0;i<enableElements.length;i++)
   {
@@ -112,7 +114,7 @@ var stateListener = {
 
 	ComposeProcessDone: function(aResult) {
     dump("\n RECEIVE ComposeProcessDone\n\n");
-    windowLocked = false;
+    gWindowLocked = false;
     CommandUpdate_MsgCompose();
     enableEditableFields();
 
@@ -224,6 +226,7 @@ var defaultController =
       case "cmd_saveAsTemplate":
       case "cmd_sendButton":
       case "cmd_sendNow":
+      case "cmd_sendWithCheck":
       case "cmd_sendLater":
 //      case "cmd_printSetup":
       case "cmd_print":
@@ -335,9 +338,10 @@ var defaultController =
       case "cmd_sendLater":
 //      case "cmd_printSetup":
       case "cmd_print":
-        return !windowLocked;
+      case "cmd_sendWithCheck":
+        return !gWindowLocked;
       case "cmd_sendNow":
-        return !(windowLocked || (ioService && ioService.offline))
+        return !(gWindowLocked || isOffline);
       case "cmd_quit":
         return true;
 
@@ -374,7 +378,7 @@ var defaultController =
 
       //Options Menu
       case "cmd_selectAddress":
-        return !windowLocked;
+        return !gWindowLocked;
       case "cmd_spelling":
         return !focusedElement;
       case "cmd_outputFormat":
@@ -461,6 +465,7 @@ var defaultController =
         }
         break;
       case "cmd_sendNow"            : if (defaultController.isCommandEnabled(command)) SendMessage();          break;
+      case "cmd_sendWithCheck"   : if (defaultController.isCommandEnabled(command)) SendMessageWithCheck();          break;
       case "cmd_sendLater"          : if (defaultController.isCommandEnabled(command)) SendMessageLater();     break;
 //      case "cmd_printSetup"         : dump("PRINT SETUP\n");                                                   break;
       case "cmd_print"              : DoCommandPrint();                                                        break;
@@ -574,16 +579,28 @@ function MessageComposeOfflineStateChanged(goingOffline)
 {
   try {
     var sendButton = document.getElementById("button-send");
+    var sendNowMenuItem = document.getElementById("menu-item-send-now");
+
+    if (!gSavedSendNowKey) {
+      gSavedSendNowKey = sendNowMenuItem.getAttribute('key');
+    }
+
+    // don't use goUpdateCommand here ... the defaultController might not be installed yet
+    goSetCommandEnabled("cmd_sendNow", defaultController.isCommandEnabled("cmd_sendNow"));
 
     if (goingOffline)
     {
       sendButton.label = sendButton.getAttribute('later_label');
       sendButton.setAttribute('tooltiptext', sendButton.getAttribute('later_tooltiptext'));
+      sendNowMenuItem.removeAttribute('key');
     }
     else
     {
       sendButton.label = sendButton.getAttribute('now_label');
       sendButton.setAttribute('tooltiptext', sendButton.getAttribute('now_tooltiptext'));
+      if (gSavedSendNowKey) {
+        sendNowMenuItem.setAttribute('key', gSavedSendNowKey);
+      }
     }
 
   } catch(e) {}
@@ -644,6 +661,7 @@ function setupLdapAutocompleteSession()
     var autocompleteLdap = false;
     var autocompleteDirectory = null;
     var prevAutocompleteDirectory = currentAutocompleteDirectory;
+    var i;
 
     autocompleteLdap = prefs.GetBoolPref("ldap_2.autoComplete.useDirectory");
     if (autocompleteLdap)
@@ -698,7 +716,7 @@ function setupLdapAutocompleteSession()
             }
             LDAPSession.serverURL = serverURL;
 
-            // don't search on strings shorter than this
+            // don't search on non-CJK strings shorter than this
             //
             try { 
                 LDAPSession.minStringLength = prefs.GetIntPref(
@@ -708,6 +726,16 @@ function setupLdapAutocompleteSession()
                 // nsLDAPAutoCompleteSession use its default.
             }
 
+            // don't search on CJK strings shorter than this
+            //
+            try { 
+                LDAPSession.cjkMinStringLength = prefs.GetIntPref(
+                  autocompleteDirectory + ".autoComplete.cjkMinStringLength");
+            } catch (ex) {
+                // if this pref isn't there, no big deal.  just let
+                // nsLDAPAutoCompleteSession use its default.
+            }
+
             // we don't try/catch here, because if this fails, we're outta luck
             //
             var ldapFormatter = Components.classes[
@@ -826,9 +854,19 @@ function setupLdapAutocompleteSession()
                 // if we make it here, we know that session initialization has
                 // succeeded; add the session for all recipients, and 
                 // remember that we've done so
-                for (var i=1; i <= MAX_RECIPIENTS; i++)
-                    document.getElementById("msgRecipient#" + i).
-                        addSession(LDAPSession);
+                var autoCompleteWidget;
+                for (i=1; i <= MAX_RECIPIENTS; i++)
+                {
+                    autoCompleteWidget = document.getElementById("msgRecipient#" + i);
+                    if (autoCompleteWidget)
+                    {
+                      autoCompleteWidget.addSession(LDAPSession);
+                      // ldap searches don't insert a default entry with the default domain appended to it
+                      // so reduce the minimum results for a popup to 2 in this case. 
+                      autoCompleteWidget.minResultsForPopup = 2;
+
+                    }
+                 }
                 sessionAdded = true;
             }
         }
@@ -840,7 +878,7 @@ function setupLdapAutocompleteSession()
         currentAutocompleteDirectory = null;
       }
       if (gLDAPSession && sessionAdded) {
-        for (var i=1; i <= MAX_RECIPIENTS; i++) 
+        for (i=1; i <= MAX_RECIPIENTS; i++) 
           document.getElementById("msgRecipient#" + i).
               removeSession(gLDAPSession);
         sessionAdded = false;
@@ -879,7 +917,7 @@ function DoCommandPreferences()
 
 function ToggleWindowLock()
 {
-  windowLocked = !windowLocked;
+  gWindowLocked = !gWindowLocked;
   CommandUpdate_MsgCompose();
 }
 
@@ -1151,8 +1189,8 @@ function ComposeLoad()
   AddMessageComposeOfflineObserver();
   AddDirectoryServerObserver(true);
 
-  if (DEBUG && msgComposeService)
-    msgComposeService.TimeStamp("Start Initializing the compose window (ComposeLoad)", false);
+  if (gLogComposePerformance)
+    msgComposeService.TimeStamp("Start initializing the compose window (ComposeLoad)", false);
   gComposeMsgsBundle = document.getElementById("bundle_composeMsgs");
 
   try {
@@ -1191,8 +1229,8 @@ function ComposeLoad()
     return;
   }
   window.tryToClose=ComposeCanClose;
-  if (DEBUG && msgComposeService)
-    msgComposeService.TimeStamp("Done with the initialization (ComposeLoad). Waiting on editor to load about::blank", false);
+  if (gLogComposePerformance)
+    msgComposeService.TimeStamp("Done with the initialization (ComposeLoad). Waiting on editor to load about:blank", false);
 }
 
 function ComposeUnload()
@@ -1407,7 +1445,7 @@ function GenericSendMessage( msgType )
 				}
 			}
 			try {
-        windowLocked = true;
+        gWindowLocked = true;
         CommandUpdate_MsgCompose();
         disableEditableFields();
 
@@ -1421,7 +1459,7 @@ function GenericSendMessage( msgType )
 			}
 			catch (ex) {
         dump("failed to SendMsg: " + ex + "\n");
-        windowLocked = false;
+        gWindowLocked = false;
         enableEditableFields();
         CommandUpdate_MsgCompose();
 			}
@@ -1434,18 +1472,44 @@ function GenericSendMessage( msgType )
 function SendMessage()
 {
 	dump("SendMessage from XUL\n");
-  // 0 = nsMsgDeliverNow
-  // RICHIE: We should really have a way of using constants and not
-  // hardcoded numbers for the first argument
 	GenericSendMessage(msgCompDeliverMode.Now);
 }
 
+function SendMessageWithCheck()
+{
+    var warn = prefs.GetBoolPref("mail.warn_on_send_accel_key");
+
+    if (warn) {
+        var buttonPressed = {value:1};
+        var checkValue = {value:false};
+        if (!gPromptService) {
+            gPromptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService();
+            gPromptService = gPromptService.QueryInterface(Components.interfaces.nsIPromptService);
+        }
+        gPromptService.confirmEx(window, 
+              gComposeMsgsBundle.getString('sendMessageCheckWindowTitle'), 
+              gComposeMsgsBundle.getString('sendMessageCheckLabel'),
+              (gPromptService.BUTTON_TITLE_IS_STRING * gPromptService.BUTTON_POS_0) +
+              (gPromptService.BUTTON_TITLE_CANCEL * gPromptService.BUTTON_POS_1),
+              gComposeMsgsBundle.getString('sendMessageCheckSendButtonLabel'),
+              null, null,
+              gComposeMsgsBundle.getString('CheckMsg'), 
+              checkValue, buttonPressed);
+        if (!buttonPressed || buttonPressed.value != 0) {
+            return;
+        }
+        if (checkValue.value) {
+            prefs.SetBoolPref("mail.warn_on_send_accel_key", false);
+        }
+    }
+
+	GenericSendMessage(isOffline ? msgCompDeliverMode.Later
+                                 : msgCompDeliverMode.Now);
+}
+
 function SendMessageLater()
 {
 	dump("SendMessageLater from XUL\n");
-  // 1 = nsMsgQueueForLater
-  // RICHIE: We should really have a way of using constants and not
-  // hardcoded numbers for the first argument
 	GenericSendMessage(msgCompDeliverMode.Later);
 }
 
@@ -1474,9 +1538,6 @@ function SaveAsDraft()
 {
 	dump("SaveAsDraft from XUL\n");
 
-  // 4 = nsMsgSaveAsDraft
-  // RICHIE: We should really have a way of using constants and not
-  // hardcoded numbers for the first argument
   GenericSendMessage(msgCompDeliverMode.SaveAsDraft);
   defaultSaveOperation = "draft";
 }
@@ -1485,9 +1546,6 @@ function SaveAsTemplate()
 {
 	dump("SaveAsTemplate from XUL\n");
 
-  // 5 = nsMsgSaveAsTemplate
-  // RICHIE: We should really have a way of using constants and not
-  // hardcoded numbers for the first argument
   GenericSendMessage(msgCompDeliverMode.SaveAsTemplate);
   defaultSaveOperation = "template";
 }
@@ -1587,6 +1645,9 @@ function SelectAddress()
 					   toAddress:toAddress,
 					   ccAddress:ccAddress,
 					   bccAddress:bccAddress});
+  // We have to set focus to the addressingwidget because we seem to loose focus often 
+  // after opening the SelectAddresses Dialog- bug # 89950
+  AdjustFocus();
 }
 
 function queryISupportsArray(supportsArray, iid) {
@@ -1712,6 +1773,7 @@ function ComposeCanClose()
   }
   if (sendOrSaveOperationInProgress)
   {
+      var result;
 
       if (gPromptService)
       {
@@ -1720,7 +1782,7 @@ function ComposeCanClose()
         var quitButtonLabel = gComposeMsgsBundle.getString("quitComposeWindowQuitButtonLabel");
         var waitButtonLabel = gComposeMsgsBundle.getString("quitComposeWindowWaitButtonLabel");
 
-        var result = {value:0};
+        result = {value:0};
         gPromptService.confirmEx(window, promptTitle, promptMsg,
           (gPromptService.BUTTON_TITLE_IS_STRING*gPromptService.BUTTON_POS_0) +
           (gPromptService.BUTTON_TITLE_IS_STRING*gPromptService.BUTTON_POS_1),
@@ -1745,7 +1807,7 @@ function ComposeCanClose()
 		window.focus();
 		if (gPromptService)
 		{
-            var result = {value:0};
+            result = {value:0};
             gPromptService.confirmEx(window,
                               gComposeMsgsBundle.getString("saveDlogTitle"),
                               gComposeMsgsBundle.getString("saveDlogMessage"),
@@ -1956,10 +2018,11 @@ function AttachVCard()
 
 function DetermineHTMLAction(convertible)
 {
+    var obj;
     if (! msgCompose.composeHTML)
     {
         try {
-            var obj = new Object;
+            obj = new Object;
             msgCompose.CheckAndPopulateRecipients(true, false, obj);
         } catch(ex) { dump("msgCompose.CheckAndPopulateRecipients failed: " + ex + "\n"); }
         return msgCompSendFormat.PlainText;
@@ -1975,7 +2038,7 @@ function DetermineHTMLAction(convertible)
 
         //Check the address book for the HTML property for each recipient
         try {
-            var obj = new Object;
+            obj = new Object;
             preferFormat = msgCompose.CheckAndPopulateRecipients(true, true, obj);
             noHtmlRecipients = obj.value;
         } catch(ex)
@@ -2028,8 +2091,8 @@ function DetermineHTMLAction(convertible)
 	  else
 	  {
 		  try {
-        var obj = new Object;
-			  msgCompose.CheckAndPopulateRecipients(true, false, obj);
+                       obj = new Object;
+                       msgCompose.CheckAndPopulateRecipients(true, false, obj);
 		  } catch(ex) { dump("msgCompose.CheckAndPopulateRecipients failed: " + ex + "\n"); }
 	  }
 
@@ -2201,7 +2264,7 @@ var attachmentBucketObserver = {
       var rawData = aData.data;
       switch (aData.flavour.contentType) {
       case "text/x-moz-url":
-      case "text/nsmessage":
+      case "text/nsmessageOrFolder":
         var separator = rawData.indexOf("\n");
         if (separator != -1) {
           prettyName = rawData.substr(separator+1);
@@ -2244,7 +2307,7 @@ var attachmentBucketObserver = {
     {
       var flavourSet = new FlavourSet();
       flavourSet.appendFlavour("text/x-moz-url");
-      flavourSet.appendFlavour("text/nsmessage");
+      flavourSet.appendFlavour("text/nsmessageOrFolder");
       flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
       return flavourSet;
     }
